#! wish
# file: $RCSfile: whomp-ds9.tcl,v $
# rcsid: $Id: whomp-ds9.tcl,v 1.2 2010/11/18 16:44:58 jwp Exp $
#
# whirc whomp
#
# ds9 support
#

namespace eval ds9 {

# format scalar degrees
# this is a knock-off of the jwp-times proc; we need the : delimiter
proc fmt_d { d } {
	set d [expr double($d)];		# enforce type

	set sign "+";
	if {$d < 0.0} {
		set sign "-";
		set d [expr abs($d)];
	}
	set deg [expr int(floor($d))];
	set m [expr ($d - floor($d)) * 60.0];
	set min [expr int(floor($m))];
	set s [expr ($m - floor($m)) * 60.0];

	set ipart [expr int($s)];
	set fpart [expr int(1e3 * ($s - $ipart))];

	set p [format "%s%02d:%02d:%02d.%03d" $sign $deg $min $ipart $fpart];

	return $p;
}

# format scalar hours
# this is a knock-off of the jwp-times proc; we need the : delimiter
proc fmt_h { h } {
	set h [expr double($h)];		# enforce type

	set h [times::h2h $h];

	set sign " ";		# no plus sign for hours
	if {$h < 0.0} {
		set sign "-";
		set h [expr abs($h)];
	}

	set hrs [expr int(floor($h))];
	set m [expr ($h - floor($h)) * 60.0];
	set min [expr int(floor($m))];
	set s [expr ($m - floor($m)) * 60.0];

	set ipart [expr int($s)];
	set fpart [expr int(1e3 * ($s - $ipart))];

	set p [format "%s%02d:%02d:%02d.%03d" $sign $hrs $min $ipart $fpart];

	return $p;
}

proc fetch { name ra dec w h } {
	# enforce types
	set ra [fmt_h [times::r2h $ra]];
	set dec [fmt_d [times::r2d $dec]];

	# decline to fix the survey; let the user's choice reign
	#exec xpaset -p ds9 dss survey dss;

	# this forces some sort of name search
	#exec xpaset -p ds9 dss name $name;

	puts stdout "exec xpaset -p ds9 frame delete;"
	exec xpaset -p ds9 frame delete;

	puts stdout "exec xpaset -p ds9 dss size $w $h;"
	exec xpaset -p ds9 dss size $w $h arcmin;

	# this forces the fetch
	puts stdout "exec xpaset -p ds9 dss coord $ra $dec sexagesimal;"
	exec xpaset -p ds9 dss coord $ra $dec sexagesimal;

	puts stdout "exec xpaset -p ds9 regions delete all;"
	exec xpaset -p ds9 regions delete all;

	puts stdout "exec xpaset -p ds9 scale linear;"
	exec xpaset -p ds9 scale linear;

	puts stdout "exec xpaset -p ds9 cmap invert;"
	exec xpaset -p ds9 cmap invert;

	# this appears to be a no-op
	puts stdout "exec xpaset -p ds9 regions color red;"
	exec xpaset -p ds9 regions color red;

	return;
}

proc show { ra dec roll w h } {
	# enforce types
	set ra [fmt_h [times::r2h $ra]];
	set dec [fmt_d [times::r2d $dec]];
	set roll [times::r2d $roll];

	puts stdout "ds9::show: ra: $ra";
	puts stdout "ds9::show: dec: $dec";
	puts stdout "ds9::show: roll: $roll";

	# ds9's z-axis is opposite my into-the-sky convention
	set roll [expr -1 * $roll];

	exec echo "box $ra $dec $w\' $h\' $roll # color = red" | xpaset ds9 regions;

	return;
}

proc test { args } {
	set ra "00:42:44";
	set dec "+41:16:08";
	set ra "00:00:00";
	set dec "+90:00:00";
	set roll 45;

	ds9::fetch m31 $ra $dec 30 30;

	ds9::show $ra $dec $roll 3 2;
	ds9::show $ra $dec 0 3 2;
	ds9::show $ra $dec 90 3 2;

	return;
}

# end of namespace ds9
}
